import django
from django.test import override_settings
import pytest
from imagekit.utils import get_storage


def test_get_storage_default():
    from django.core.files.storage import default_storage

    if django.VERSION >= (4, 2):
        assert get_storage() == default_storage
    else:
        assert isinstance(get_storage(), type(default_storage._wrapped))


@pytest.mark.skipif(
    django.VERSION >= (5, 1),
    reason="DEFAULT_FILE_STORAGE is removed in Django 5.1.",
)
def test_get_storage_custom_import_path():
    from tests.utils import CustomStorage

    with override_settings(IMAGEKIT_DEFAULT_FILE_STORAGE="tests.utils.CustomStorage"):
        assert isinstance(get_storage(), CustomStorage)


@pytest.mark.skipif(
    django.VERSION < (4, 2),
    reason="STORAGES was introduced in Django 4.2",
)
def test_get_storage_custom_key():
    from tests.utils import CustomStorage

    with override_settings(
        STORAGES={
            "custom": {
                "BACKEND": "tests.utils.CustomStorage",
            }
        },
        IMAGEKIT_DEFAULT_FILE_STORAGE="custom",
    ):
        assert isinstance(get_storage(), CustomStorage)



"""
Make sure that the various IK classes can be successfully serialized and
deserialized. This is important when using IK with Celery.

"""
import pytest

from imagekit.cachefiles import ImageCacheFile

from .imagegenerators import TestSpec
from .utils import (clear_imagekit_cache, create_photo, get_unique_image_file,
                    pickleback)


@pytest.mark.django_db(transaction=True)
def test_imagespecfield():
    clear_imagekit_cache()
    instance = create_photo('pickletest2.jpg')
    thumbnail = pickleback(instance.thumbnail)
    thumbnail.generate()


@pytest.mark.django_db(transaction=True)
def test_circular_ref():
    """
    A model instance with a spec field in its dict shouldn't raise a KeyError.

    This corresponds to #234

    """
    clear_imagekit_cache()
    instance = create_photo('pickletest3.jpg')
    instance.thumbnail  # Cause thumbnail to be added to instance's __dict__
    pickleback(instance)


def test_cachefiles():
    clear_imagekit_cache()
    with get_unique_image_file() as source_file:
        spec = TestSpec(source=source_file)
        file = ImageCacheFile(spec)
        file.url
        # remove link to file from spec source generator
        # test __getstate__ of ImageCacheFile
        file.generator.source = None
        restored_file = pickleback(file)
        assert file is not restored_file
        # Assertion for #437 and #451
        assert file.storage is restored_file.storage



from hashlib import md5
from unittest import mock
import os

import pytest
from django.conf import settings

from imagekit.cachefiles import ImageCacheFile, LazyImageCacheFile
from imagekit.cachefiles.backends import Simple

from .imagegenerators import TestSpec
from .utils import (DummyAsyncCacheFileBackend, assert_file_is_falsy,
                    assert_file_is_truthy, get_image_file,
                    get_unique_image_file)


def test_no_source_falsiness():
    """
    Ensure cache files generated from sourceless specs are falsy.

    """
    spec = TestSpec(source=None)
    file = ImageCacheFile(spec)
    assert_file_is_falsy(file)


def test_sync_backend_truthiness():
    """
    Ensure that a cachefile with a synchronous cache file backend (the default)
    is truthy.

    """
    with get_unique_image_file() as source_file:
        spec = TestSpec(source=source_file)
        file = ImageCacheFile(spec)
        assert_file_is_truthy(file)


def test_async_backend_falsiness():
    """
    Ensure that a cachefile with an asynchronous cache file backend is falsy.

    """
    with get_unique_image_file() as source_file:
        spec = TestSpec(source=source_file)
        file = ImageCacheFile(spec, cachefile_backend=DummyAsyncCacheFileBackend())
        assert_file_is_falsy(file)


def test_no_source_error():
    spec = TestSpec(source=None)
    file = ImageCacheFile(spec)
    with pytest.raises(TestSpec.MissingSource):
        file.generate()


def test_repr_does_not_send_existence_required():
    """
    Ensure that `__repr__` method does not send `existance_required` signal

    Cachefile strategy may be configured to generate file on
    `existance_required`.
    To generate images, backend passes `ImageCacheFile` instance to worker.
    Both celery and RQ calls `__repr__` method for each argument to enque call.
    And if `__repr__` of object will send this signal, we will get endless
    recursion

    """
    with mock.patch('imagekit.cachefiles.existence_required') as signal:
        # import here to apply mock
        from imagekit.cachefiles import ImageCacheFile

        with get_unique_image_file() as source_file:
            spec = TestSpec(source=source_file)
            file = ImageCacheFile(
                spec,
                cachefile_backend=DummyAsyncCacheFileBackend()
            )
            file.__repr__()
            assert signal.send.called is False


def test_memcached_cache_key():
    """
    Ensure the default cachefile backend is sanitizing its cache key for
    memcached by default.

    """

    class MockFile:
        def __init__(self, name):
            self.name = name

    backend = Simple()
    extra_char_count = len('state-') + len(settings.IMAGEKIT_CACHE_PREFIX)

    length = 199 - extra_char_count
    filename = '1' * length
    file = MockFile(filename)
    assert backend.get_key(file) == '%s%s-state' % (settings.IMAGEKIT_CACHE_PREFIX, file.name)

    length = 200 - extra_char_count
    filename = '1' * length
    file = MockFile(filename)
    assert backend.get_key(file) == '%s%s:%s' % (
        settings.IMAGEKIT_CACHE_PREFIX,
        '1' * (200 - len(':') - 32 - len(settings.IMAGEKIT_CACHE_PREFIX)),
        md5(('%s%s-state' % (settings.IMAGEKIT_CACHE_PREFIX, filename)).encode('utf-8')).hexdigest())


def test_lazyfile_stringification():
    file = LazyImageCacheFile('testspec', source=None)
    assert str(file) == ''
    assert repr(file) == '<ImageCacheFile: None>'

    with get_image_file() as source_file:
        file = LazyImageCacheFile('testspec', source=source_file)
    file.name = 'a.jpg'
    assert str(file) == 'a.jpg'
    assert repr(file) == '<ImageCacheFile: a.jpg>'


def test_generate_file_already_exists(caplog):
    with get_unique_image_file() as source_file:
        spec = TestSpec(source=source_file)
        file_1 = ImageCacheFile(spec)
        file_1._generate()
        # generate another cache image with the same name
        file_2 = ImageCacheFile(spec, name=file_1.name)
        file_2._generate()

    assert len(caplog.records) == 1
    storage, name, actual_name, cachefile_backend = caplog.records[0].args
    assert storage == file_2.storage
    assert name == file_2.name
    assert actual_name != name
    assert os.path.basename(actual_name) in storage.listdir(os.path.dirname(actual_name))[1]
    assert cachefile_backend == file_2.cachefile_backend



from .processors import Thumbnail as ThumbnailProcessor
from .registry import register
from .specs import ImageSpec
from django.conf import settings

default_thumbnail_format = getattr(settings, 'IMAGEKIT_DEFAULT_THUMBNAIL_FORMAT', None)

class Thumbnail(ImageSpec):
    def __init__(self, width=None, height=None, anchor=None, crop=None, upscale=None, format=default_thumbnail_format, **kwargs):
        self.processors = [ThumbnailProcessor(width, height, anchor=anchor,
                                              crop=crop, upscale=upscale)]
        super().__init__(**kwargs)
        self.format = format


register.generator('imagekit:thumbnail', Thumbnail)



from . import conf, generatorlibrary
from .pkgmeta import *
from .registry import register, unregister
from .specs import ImageSpec

__all__ = [
    'ImageSpec', 'conf', 'generatorlibrary', 'register', 'unregister',
    '__title__', '__author__', '__version__', '__license__'
]



#!/usr/bin/env python
import codecs
import os
import sys

from setuptools import find_packages, setup


if 'publish' in sys.argv:
    os.system('python3 -m build')
    os.system('python3 -m twine upload --repository django_imagekit dist/*')
    sys.exit()


def read(filepath):
    with codecs.open(filepath, 'r', 'utf-8') as f:
        return f.read()


def exec_file(filepath, globalz=None, localz=None):
    exec(read(filepath), globalz, localz)


# Load package meta from the pkgmeta module without loading imagekit.
pkgmeta = {}
exec_file(
    os.path.join(os.path.dirname(__file__), 'imagekit', 'pkgmeta.py'),
    pkgmeta
)


setup(
    name='django-imagekit',
    version=pkgmeta['__version__'],
    description='Automated image processing for Django models.',
    long_description=read(os.path.join(os.path.dirname(__file__), 'README.rst')),
    long_description_content_type='text/x-rst',
    author='Matthew Tretter',
    author_email='m@tthewwithanm.com',
    maintainer='Venelin Stoykov',
    maintainer_email='venelin.stoykov@industria.tech',
    license='BSD',
    url='http://github.com/matthewwithanm/django-imagekit/',
    packages=find_packages(exclude=['*.tests', '*.tests.*', 'tests.*', 'tests']),
    zip_safe=False,
    include_package_data=True,
    install_requires=[
        'django-appconf',
        'pilkit',
    ],
    extras_require={
        'async': ['django-celery>=3.0'],
        'async_rq': ['django-rq>=0.6.0'],
        'async_dramatiq': ['django-dramatiq>=0.4.0'],
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.13',
        'Programming Language :: Python :: 3.14',
        'Topic :: Utilities'
    ],
)



from pilkit.processors import *

__all__ = [
    # Base
    'ProcessorPipeline', 'Adjust', 'Reflection', 'Transpose',
    'Anchor', 'MakeOpaque',
    # Crop
    'TrimBorderColor', 'Crop', 'SmartCrop',
    # Resize
    'Resize', 'ResizeToCover', 'ResizeToFill', 'SmartResize',
    'ResizeCanvas', 'AddBorder', 'ResizeToFit', 'Thumbnail'
]
